@@ -2,30 +2,31 @@ package CHI::Driver::CacheCache;
 use Cache::Cache;
 use Carp;
 use CHI::Util qw(require_dynamic);
-use Hash::MoreUtils qw(slice_exists);
 use Moose;
 use strict;
 use warnings;
 
-extends 'CHI::Driver::Base::CacheContainer';
+extends 'CHI::Driver';
+
+with 'CHI::Driver::Role::CacheContainer';
 
 has 'cc_class'   => ( is => 'ro', isa => 'Str',     required => 1 );
 has 'cc_options' => ( is => 'ro', isa => 'HashRef', required => 1 );
 
 __PACKAGE__->meta->make_immutable();
 
-sub BUILD {
-    my ( $self, $params ) = @_;
+sub _build_contained_cache {
+    my ($self) = @_;
 
     my $cc_class   = $self->{cc_class};
     my $cc_options = $self->{cc_options};
-    my %subparams  = slice_exists( $params, 'namespace' );
+    my %subparams  = ( namespace => $self->namespace );
 
     require_dynamic($cc_class);
 
     my %final_cc_params = ( %subparams, %{$cc_options} );
-    $self->{_contained_cache} = $self->{cc_cache} =
-      $cc_class->new( \%final_cc_params );
+
+    return $cc_class->new( \%final_cc_params );
 }
 
 1;
